{% extends "_base.html" %}
{% block title %} main page {% endblock %}
{% block content %}

<div class="row">
    <div class="col-lg-4">
        {% for link,name in links.items() %}
        <a href="{{link}}" class="reference-link">
            <h4>{{name}}</h4>
        </a>
        {% endfor %}
    </div>
    </div>
    <div class="row">
        <div class="col-lg-12" id="reference-content">
        </div>
    </div>
</div>


{% endblock %}

