"""

"""
import unittest


class NoseInvalidDriverTest(unittest.TestCase):
    def test_driver_single(self):
        self.driver.get("http://never.com")
    def test_driver_double(self):
        self.drivers[0].get("http://never.com")
        self.drivers[1].get("http://never.com")
