{% extends "_base.html" %}
{% block title %}login{% endblock %}
{% block content %}
<div class="row">
    <div class="col-md-8 col-md-offset-2">
        <form role="form" method="POST" name="login">
            <div>
                <label for="email">Email address</label>
                <input class='form-control' id="email" name="email" type="text">
            </div>
            <div>
                <label for="email">Password</label>
                <input class='form-control' id="password" name="password" type="password">
            </div>
            <input id="login-submit" class='btn btn-primary form-control' id="submit" type="submit" value="Login">
        </form>
    </div>
</div>
{% endblock %}
